"""
Functions to evaluate other functions.
"""

import inspect
import typing
import logging
from collections import OrderedDict

logger=logging.getLogger("__main__")

def getParamDocStr(func:typing.Callable) ->dict:
    """
    For a given function, evaluate it's dosctring and return the docstrings for individual parameters. The docstrings
    for parameters must be in form ':param something: whatever' .

    Usage:
        def something(a,b):
             "":param a: This is something
               :param b: This is nothing.
            ""
            return a+B

        >>> getParamDocStr(something)
        # Will return {"a":"This something","b":"This is nothing"}
    :param func: This can be a class or function whose docstring is to be analyzed.
    :return:
    """
    doc=inspect.getdoc(func)

    #Assumptions are that docstrings for parameters start with :param and may continue to the next line with single
    # line-break. So, if a :param is encountered, start recording a docstring,and keep recording till next :param is reached.

    paramDict = {}
    if doc:
        docLines=doc.strip().split("\n")

        paramName=None
        currentParamVal=None
        for line in docLines:
            lineStrip=line.strip()
            if lineStrip.startswith(":param"):

                if paramName and currentParamVal:
                    paramDict[paramName]=currentParamVal
                    paramName,currentParamVal=None,None

                lineStripSplit=lineStrip.split(":")
                try:
                    if lineStripSplit[1].startswith("param"):
                        paramName=lineStripSplit[1].split()[1]
                        currentParamVal=" ".join(lineStripSplit[2:])
                        paramDict[paramName]=currentParamVal
                except:
                    pass
            elif lineStrip and currentParamVal and not lineStrip.startswith(":"):
                currentParamVal+=" "+lineStrip

    return paramDict

def getParamNameTypeDefaults(func:typing.Callable,guessAnnotationFromDefault=True) ->dict:
    """
    For a given function, return parameters, their types and default values.


    Usage:
        def something(a:float=3,b=2):
             "":param a: This is something
               :param b: This is nothing.
            ""
            return a+B

        >>> getParamNameTypeDefaults(something)
        # Will return OrderedDict([('a', {'default': 3, 'annotation': <class 'float'>}), ('b', {'default': 2, 'annotation': <class 'int'>})])

    :param func: The function whose parameters etc is to be returned.
    :return:
    """
    funcSig=inspect.signature(func)

    funcDict=OrderedDict()

    for name,val in funcSig.parameters.items():

        annotationFromDefault=None
        if val.default != inspect._empty:
            annotationFromDefault=type(val.default)
            defaultVal = val.default
        else:
            defaultVal=None


        if val.annotation==inspect._empty:
            if guessAnnotationFromDefault:
                annotation=annotationFromDefault
                logger.debug("Guessed annotation from default value as %s"%annotation)
            else:
                annotation=None
        else:
            annotation=val.annotation


        funcDict[val.name]={"default":defaultVal,"annotation":annotation}

    return funcDict

class _FunctionParameter(object):
    """This class is meaant to provide a clean way to access the various aspects of a function parameter"""
    def __init__(self,paramName,paramDefault,paramType,paramDocstring):
        """

        :param paramName: Name of the parameter/variable.
        :param paramDefault: Dedfault value.
        :param paramType: Type of the parameter.
        :param paramDocstring: Docstring for the parameter.
        """
        self.paramName=paramName
        self.paramDefault=paramDefault
        self.paramType=paramType
        self.paramDocstring=paramDocstring

    def __repr__(self):
        return "Parameter details for %s"%self.paramName

    def __str__(self):
        return "Name: %s, Default: %s, Type: %s Docstring: %s"%(self.paramName,self.paramDefault,self.paramType,self.paramDocstring)

def getFunctionDetails(func,guessAnnotationFromDefault:bool=True) -> typing.List[_FunctionParameter]:
    """
    For a given function analyze its parameters and docstrings and return a list of _FunctionParameter instances that
    contain details regarding each parameter.

    Usage:
        def something(a:float=3,b=2):
             "":param a: This is something
               :param b: This is nothing.
            ""
            return a+B

        >>> l=getFunctionDetails(something)
        >>> print(l)
        # Will print: [Parameter details for a, Parameter details for b]
        >>> for value in l:
        >>>    print(value)
        # Will print:
        #   Name: a, Default: 3, Type: <class 'float'> Docstring:  This is something
        #   Name: b, Default: 2, Type: <class 'int'> Docstring:  This is nothing.

    :param func: The function whose parameters are to be studied.

    :return:
    """

    paramList=[]

    docStrings=getParamDocStr(func)
    defaultValType=getParamNameTypeDefaults(func,guessAnnotationFromDefault=guessAnnotationFromDefault)

    for key,val in defaultValType.items():
        paramName,paramDefault,paramType=key,val["default"],val["annotation"]
        if key in docStrings:
            paramDoc=docStrings[key]
        else:
            paramDoc=""

        paramList.append(_FunctionParameter(paramName,paramDefault,paramType,paramDoc))

    return paramList



if __name__ == "__main__":
    from sarUtil.textTools.strings import resizeString
    from sarUtil.multiProc.mappers import poolStarMapper, poolMapper
    o=getFunctionDetails(resizeString)


    for value in o:
        print(value)