
import logging
import os



def assertEqual(variableName,variable,expectedValue,logger=None,loggingLevel=logging.DEBUG):
    """
     A simple assertion test to check if a certain variable is equal to an expected value.

     Usage:
        from sarUtil.log import LoggerMain
        logger=LoggerMain()
        something=32

        assertEqual("something",something, 32,logger) #Check if something is equal to 32
        >> 17:59:13 "DEBUG"|mod:assertions|line:33|func:testAssertionEqual|msg:Assertion test passed. The value for something(32) was equal to 32.

        assertEqual("something",something, 31,logger)
        >> Traceback (most recent call last):
        >>  AssertionError: The value for 'something'(32) should be equal to 31.


    :param variableName: A string that names the variable.
    :param variable: The actual variable.
    :param expectedValue: The expected value of the variable that the actual value will be tested against.
    :param logger: Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger
    :param loggingLevel: Defaults to logging.DEBUG
    :return:
    """
    assert variable==expectedValue,"The value for '%s'(%s) should be equal to %s."%(variableName,variable,expectedValue)
    if logger:
        msg="The value for %s(%s) was equal to %s."%(variableName,variable,expectedValue)
        logger.log(loggingLevel,"Assertion test passed. %s"%msg)

    return True

def assertSame(variableName,variable,expectedValue,logger=None,loggingLevel=logging.DEBUG):
    """
     A simple assertion test to check if a certain variable is the same an expected value. Read the documentation for testAssertionEqual to see usage.

    :param variableName: A string that names the variable.
    :param variable: The actual variable.
    :param expectedValue: The expected value of the variable that the actual value will be tested against
    :param logger: Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger
    :param loggingLevel: Defaults to logging.DEBUG
    :return:
    """

    assert variable is expectedValue, "The value for '%s'(%s) should be same as %s." % (
    variableName, variable, expectedValue)
    if logger:
        msg = "The value for %s(%s) is the same as %s." % (variableName, variable, expectedValue)
        logger.log(loggingLevel, "Assertion test passed. %s" % msg)

    return True

def assertIsWithin(variableName,variable,expectedValues,pickFirstLastForMessage=False,logger=None,loggingLevel=logging.DEBUG):
    """
    A simple assertion test to check if a certain variable within a range of expected values. Read the documentation for testAssertionEqual to see usage.

    :param variableName: A string that names the variable.
    :param variable: The actual variable.
    :param expectedValues: A sequence of values that represent the possible correct choices for the value of the variable.
    :param pickFirstLastForMessage: If set to True, then only the first and last value of from the choices will be displayed in
        the logging and error messages. This makes sense, if the possible number of values are too many.
    :type pickFirstLastForMessage: bool
    :param logger:  Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger

    :param loggingLevel:  Defaults to logging.DEBUG
    :return:
    """
    if pickFirstLastForMessage:
        msgStr="The value for '%s'(%s) should be one of the %s values between %s and %s"%(variableName,variable,len(expectedValues),expectedValues[0],expectedValues[-1])
    else:
        valuesString = ",".join([str(val) for val in expectedValues])
        msgStr="The value for '%s'(%s) should be one among the following: %s"%(variableName,variable,valuesString)

    assert variable in expectedValues,msgStr

    if logger:
        logger.log(loggingLevel,"The following assertion test was passed: %s"%msgStr)

    return True

def assertPathExists(path,logger=None,loggingLevel=logging.DEBUG):
    """
    Given an absolute path, assert that it exists.
    :param path: The path that needs to be checked.

    :param logger:  Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger

    :param loggingLevel:  Defaults to logging.DEBUG
    :return: True if the path exists, or else crash!
    """

    assert(os.path.exists(path)),"The path %s does not exist."%path

    if logger:
        msg="The path %s exists."%path
        logger.log(loggingLevel,"The following assertion test was passed: %s"%msg)

    return True

def assertDirExists(dirPath,logger=None,loggingLevel=logging.DEBUG):
    """
    Given an absolute path, assert that it exists and that it is a directory.
    :param dirPath: The directory path that needs to be checked.

    :param logger:  Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger

    :param loggingLevel:  Defaults to logging.DEBUG
    :return: True if the path exists, or else crash!
    """

    assert (os.path.exists(dirPath)), "The directory path %s does not exist." % dirPath

    assert os.path.isdir(dirPath),"The path %s is not a directory."%dirPath

    if logger:
        msg = "The path %s is a directory." % dirPath
        logger.log(loggingLevel, "The following assertion test was passed: %s" % msg)

    return True

def assertFileExists(filePath,logger=None,loggingLevel=logging.DEBUG):
    """
    Given an absolute path for a directory, assert that it exists and that it is a directory.
    :param filePath: The file path that needs to be checked.

    :param logger:  Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger

    :param loggingLevel:  Defaults to logging.DEBUG
    :return: True if the path exists, or else crash!
    """

    assert (os.path.exists(filePath)), "The file path %s does not exist." % filePath

    assert os.path.isfile(filePath), "The path %s is not a file." % filePath

    if logger:
        msg = "The path %s is a file." % filePath
        logger.log(loggingLevel, "The following assertion test was passed: %s" % msg)

    return True

def assertDirEmpty(dirPath,logger=None,loggingLevel=logging.DEBUG):
    """
    Given an absolute path for a directory, assert that it exists and is empty
    :param dirPath: The file path that needs to be checked.

    :param logger:  Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger

    :param loggingLevel:  Defaults to logging.DEBUG
    :return: True if empty, or else crash!
    """

    assert (os.path.exists(dirPath)), "The directory path %s does not exist." % dirPath

    assert os.path.isdir(dirPath), "The path %s is not a directory." % dirPath

    assert not os.listdir(dirPath),"The directory %s is not empty"%dirPath

    if logger:
        msg = "The directory %s is empty" % dirPath
        logger.log(loggingLevel, "The following assertion test was passed: %s" % msg)

    return True

def assertDirNotEmpty(dirPath,logger=None,loggingLevel=logging.DEBUG):
    """
    Given an absolute path for a directory, assert that it exists and is not empty
    :param dirPath: The directory path that needs to be checked.

    :param logger:  Optional logger, if provided then a successful assertion pass will be logged by this.
    :type logger: logging.Logger

    :param loggingLevel:  Defaults to logging.DEBUG
    :return: True if not empty, or else crash!
    """

    assert (os.path.exists(dirPath)), "The directory path %s does not exist." % dirPath

    assert os.path.isdir(dirPath), "The path %s is not a directory." % dirPath

    assert os.listdir(dirPath), "The directory %s is empty" % dirPath

    if logger:
        msg = "The directory %s is not empty" % dirPath
        logger.log(loggingLevel, "The following assertion test was passed: %s" % msg)

    return True

if __name__=="__main__":
    pass







