"""
=======================
Date: 31st August 2018
=======================

Some functions to make the reading of json and csv files easy and no nonsense. The following functions are available:

1. readCsvReturnList: read a csv file path and return a list.
2. readJsonReturnData: Read a json file and return a dictionary.
3. readOnlineCsvReturnList: Read a csv file hosted somewhere online and return a list.
4. readOnlineJsonReturnData: Read a json file hosted somewhere online and return a dict.
5. jsonObjSummaryStr: Given a json object that was read from a json file, return a string summary of the
    that object.
6. jsonFileSummaryStr: Given a json file, return a string summary of the object inside the json file.
7. writeJson: Write a json file.
8. writeCsv: Write a csv file.

"""


import csv,json
import os,tempfile
import urllib.request, urllib.parse, urllib.error
import  logging
from typing import List,Dict

logger=logging.getLogger("__main__")

_typeToString = {
    bool: 'boolean',
    dict: 'dict',
    float: 'float',
    int: 'int',
    list: 'list',
    int: 'int',
    type(None): 'None',
    str: 'str',
}



#Need the header for those pages
_hdr = {'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
       'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
       'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
       'Accept-Encoding': 'none',
       'Accept-Language': 'en-US,en;q=0.8',
       'Connection': 'keep-alive'}


class JsonItem(object):
    """
    Typical usage:
        import json
        with open("somefile.json") as jsonData:
            jsonObj = json.load(jsonData)
            x = JsonItem(jsonObj, strictness="keys")
            print(x)

    """
    DEFAULT_STRICTNESS = 'length'

    def __init__(self, item, indent=0, strictness=None):
        """
        :param item: This is the item that is obtained after reading the json file and
            extracting the data from it.
        :param indent: The starting string indent.
        :param strictness: The option to determine how strictly JSON objects and lists should be compared.
            Strictness affects the amount of output you'll see. For example if a list contains two lists,
            `describejson.py` will just tell you that very concisely if you use `--strictness type`.
            If you want the fact that things differ in length or contents, use a higher strictness level.
            In order of increasing strictness, the possible values are:
                type- compare everything by type only.
                length- compare lists and objects by length.
                keys- compare lists by equality, objects by keys.
                equal- compare objects and list by equality.


        """
        self._strictness = strictness or self.DEFAULT_STRICTNESS
        self._compare = getattr(self, '_eq_%s' % self._strictness)
        self._indent = indent
        self._duplications = 1
        self._type = type(item)
        if self._type == dict:
            self._len = len(item)
            if strictness == 'keys':
                self._keys = sorted(item.keys())
            elif strictness == 'equal':
                self._item = item
            self._summarize(item.values())
        elif self._type == list:
            self._len = len(item)
            if strictness in ('keys', 'equal'):
                self._item = item
            self._summarize(item)

    def _summarize(self, items):
        self._contents = []
        for i, item in enumerate(items):
            jsonItem = JsonItem(item, self._indent + 2, self._strictness)
            if i == 0:
                self._contents.append(jsonItem)
            else:
                if self._contents[-1] == jsonItem:
                    self._contents[-1]._duplications += 1
                else:
                    self._contents.append(jsonItem)

    def __eq__(self, other):
        return self._compare(other)

    def _eq_type(self, other):
        """Equality only according to item type."""
        return self._type == other._type

    def _eq_length(self, other):
        """Dicts and lists must have the same number of keys."""
        if self._type == other._type:
            if self._type == dict or self._type == list:
                return self._len == other._len
            else:
                return True
        else:
            return False

    def _eq_keys(self, other):
        """Lists must be equal, dicts must have the same keys."""
        if self._type == other._type:
            if self._type == list:
                return self._item == other._item
            elif self._type == dict:
                return self._keys == other._keys
            else:
                return True
        else:
            return False

    def _eq_equal(self, other):
        """Lists and dicts must be equal."""
        if self._type == other._type:
            if self._type in (dict, list):
                return self._item == other._item
            else:
                return True
        else:
            return False

    def _strList(self):
        prefix = '%s%s %s%s' % (
            self._indent * ' ', self._duplications, _typeToString[self._type],
            '' if self._duplications == 1 else 's')
        if self._type in (dict, list):
            result = ['%s of length %d.%s' % (
                prefix, self._len, ' Values:' if self._len else '')]
            for item in self._contents:
                result.extend(item._strList())
        else:
            result = [prefix]
        return result

    def __str__(self):
        return '\n'.join(self._strList())

def jsonObjSummaryStr(jsonObject, strictness="type", indent=0):
    """
    Given a json object that was read from a json file, return a string summary of the
    that object.
    :param jsonObject: The json object whose summary is to be printed.
    :param indent: The starting string indent.
    :param strictness: The option to determine how strictly JSON objects and lists should be compared.
        Strictness affects the amount of output you'll see. For example if a list contains two lists,
        `describejson.py` will just tell you that very concisely if you use `--strictness type`.
        If you want the fact that things differ in length or contents, use a higher strictness level.
        In order of increasing strictness, the possible values are:
            type- compare everything by type only.
            length- compare lists and objects by length.
            keys- compare lists by equality, objects by keys.
            equal- compare objects and list by equality.
    :return: The string summary of the json object.
    """
    return str(JsonItem(jsonObject,strictness=strictness,indent=indent))

def jsonFileSummaryStr(jsonFilePath,encoding="utf-8",strictnessForSummary="type",indentForSummary=2):
    """
    Read a json file and return its summary. This can be useful for checking if a json file needs
    to be processed further or not.

    :param jsonFilePath:
    :param encoding: The encoding value that is to be passed on to the open() function.
    :param strictnessForSummary: The strictness variable for logging the json. Valid options
        are type, length, keys and equal. For details read the docstrings of the function jsonSummaryStr
    :param indentForSummary: Starting indent of the summary string.
    :return:

    """
    assert os.path.exists(jsonFilePath),"The input for jsonFilePath(%s) was not found."%jsonFilePath

    with open(jsonFilePath,encoding=encoding) as jsonData:
        outputData=json.load(jsonData)
    logger.debug("Successfully read the file %s"%jsonFilePath)


    logger.debug("Returning a json summary string.")
    jsonStr=jsonObjSummaryStr(outputData, strictness=strictnessForSummary, indent=indentForSummary)
    return jsonStr

def _downloadUrlToFile(urlLink,filePath):
    """

    :param url: Url to be downloaded. Will usually be a linke to a json, csv file.
    :param filePath: Path to write to.
    :return:
    """
    try:

        csvRequest=urllib.request.Request(urlLink,headers=_hdr)
        csvData=urllib.request.urlopen(csvRequest).read()

        with open(filePath,"wb") as csvWrite:
            csvWrite.write(csvData)
        logger.debug("Downloaded %s to %s"%(urlLink,filePath))
    except:
        logger.error("Could not download %s"%urlLink)
        raise Exception

    return filePath

def readCsvReturnList(csvFilePath,delimiter=',',encoding="utf-8"):
    """
    Take a csv file path and return a List.
    :param csvFilePath: Path of the csv file.
    :return: A list extracted from the csv.
    :rtype: List
    """
    assert os.path.exists(csvFilePath),"The input for csvFilePath(%s) was not found."%csvFilePath


    with open(csvFilePath,encoding=encoding) as csvData:
        outputList= list(csv.reader(csvData,delimiter=delimiter))
    logger.debug("Successfully extracted data from %s"%csvFilePath)

    logger.debug("Returning a list of %s values"%len(outputList))

    return outputList

def readJsonReturnData(jsonFilePath,encoding="utf-8",logDetailedJsonSummary=False,strictnessForSummary="type",indentForSummary=2):
    """
    Read a json file and return data. If logging is enabled, then the type of the data
    returned will be logged.

    :param jsonFilePath:
    :param encoding: The encoding value that is to be passed on to the open() function.
    :param logDetailedJsonSummary: If set to True, a detailed summary of the json object will be logged.
    :param strictnessForSummary: The strictness variable for logging the json. Valid options
        are type, length, keys and equal. For details read the docstrings of the function jsonSummaryStr
    :param indentForSummary: Starting indent of the summary string.
    :return:

    """

    assert os.path.exists(jsonFilePath),"The input for jsonFilePath(%s) was not found."%jsonFilePath

    with open(jsonFilePath,encoding=encoding) as jsonData:
        outputData=json.load(jsonData)
    logger.debug("Successfully read the file %s"%jsonFilePath)
    if logDetailedJsonSummary:
        jsonStr=jsonObjSummaryStr(outputData, strictness=strictnessForSummary, indent=indentForSummary)
        logger.debug("Returning a json object. Detailed summary of the json object is below.\n%s"%jsonStr)
    else:
        logger.debug("Returning an object of type %s after reading %s"%(type(outputData),jsonFilePath))

    return outputData

def readOnlineCsvReturnList(csvUrl,delimiter=',',encoding="utf-8"):
    """
    Return list from a url that contains csv data.
    :param csvUrl:
    :return:
    :rtype: List
    """

    tmpCsvFile=tempfile.mktemp(suffix=".csv")

    _downloadUrlToFile(csvUrl,tmpCsvFile)

    listVal= readCsvReturnList(tmpCsvFile,delimiter=delimiter,encoding=encoding)

    os.remove(tmpCsvFile)
    logger.debug("Deleting the temp csv file %s"%tmpCsvFile)

    return  listVal

def readOnlineJsonReturnData(jsonUrl,encoding="utf-8",logDetailedJsonSummary=False,strictnessForSummary="type",indentForSummary=2):
    """
    Return data from url that contains json data.
    :param jsonUrl: The url that contains the json data
    :param encoding: The encoding value that is to be passed on to the open() function.
    :param logDetailedJsonSummary: If set to True, a detailed summary of the json object will be logged.
    :param strictnessForSummary: The strictness variable for logging the json. Valid options
        are type, length, keys and equal. For details read the docstrings of the function jsonSummaryStr
    :param indentForSummary: Starting indent of the summary string.
    :return:
    """

    tmpJsonFile=tempfile.mktemp(suffix=".json")

    tmpJsonFile=_downloadUrlToFile(jsonUrl,tmpJsonFile)

    jsonVal= readJsonReturnData(tmpJsonFile,encoding=encoding,logDetailedJsonSummary=logDetailedJsonSummary,
                                strictnessForSummary=strictnessForSummary,indentForSummary=indentForSummary)


    os.remove(tmpJsonFile)

    return  jsonVal

def writeJson(filePath,data, encoding=None,skipkeys=False,ensure_ascii=True,check_circular=True,
              allow_nan=True, indent=None,separators=None, default=None, sort_keys=False, **kw):
    """
    Write python object 'data' into a file 'filePath'. The parameter encoding will be passed on to the open() method
    and all the other parameters will be passed to json.dump.

    Usage:
        a={"a":3,"b":5}

        writeJson(r"C:\someFolder\someFile.json",a)

    :param filePath: This is the path to which the json file is to be written.
    :param data: Data that is to be written to.
    :param encoding: Defaults to None but can be oter encoding options such as utf-8, latin-1 etc.
    :param skipkeys: If skipkeys is true (default: False), then dict keys that are not of a basic type (str, int, float,
        bool, None) will be skipped instead of raising a TypeError.
    :param ensure_ascii: If ensure_ascii is true (the default), the output is guaranteed to have all incoming non-ASCII
        characters escaped. If ensure_ascii is false, these characters will be output as-is.
    :param check_circular: If check_circular is false (default: True), then the circular reference check for container
        types will be skipped and a circular reference will result in an OverflowError (or worse).
    :param allow_nan: If allow_nan is false (default: True), then it will be a ValueError to serialize out of range
        float values (nan, inf, -inf) in strict compliance of the JSON specification. If allow_nan is true, their
        JavaScript equivalents (NaN, Infinity, -Infinity) will be used.
    :param indent: If indent is a non-negative integer or string, then JSON array elements and object members will be
        pretty-printed with that indent level. An indent level of 0, negative, or "" will only insert newlines.
        None (the default) selects the most compact representation. Using a positive integer indent indents that many
        spaces per level. If indent is a string (such as "\t"), that string is used to indent each level.
    :param separators: If specified, separators should be an (item_separator, key_separator) tuple. The default is
        (', ', ': ') if indent is None and (',', ': ') otherwise. To get the most compact JSON representation, you
        should specify (',', ':') to eliminate whitespace.
    :param default: If specified, default should be a function that gets called for objects that can’t otherwise be
        serialized. It should return a JSON encodable version of the object or raise a TypeError. If not specified,
        TypeError is raised.
    :param sort_keys: If sort_keys is true (default: False), then the output of dictionaries will be sorted by key.
    :param kw: Additional keywords that will be passed on to json.dump
    :return: The same filePath that was provided as input.
    """
    logger.debug("The provided data type is %s"%(type(data)))

    with open(filePath,"w",encoding=encoding) as jsonWriteStream:
        json.dump(data,jsonWriteStream,skipkeys=skipkeys,ensure_ascii=ensure_ascii,check_circular=check_circular,
                  allow_nan=allow_nan,indent=indent,separators=separators,default=default,sort_keys=sort_keys,**kw)

    logger.debug("Successfully wrote the data to the file %s"%filePath)

    return filePath

def writeCsv(filePath,data,dataHeaderRow=None,dialect='excel',delimiter=',',doublequote=True,escapechar=None,lineterminator='\n',
             quotechar='"',quoting=csv.QUOTE_MINIMAL,skipinitialspace=False,strict=True):
    """
    Write data into a csv file. Most of the parameters here are ported and copied over from the csv.writer method.

    Usage:
        a=[list(10) for x in range(8)]
        writeCsv(r"D:\some.csv",a)

    :param filePath: The path to which the csv is to be written to.
    :param data: The data that is to be written to the csv file.
    :param dataHeaderRow: If provided then this will be written first into the file using the writer.writerow method.
    :param dialect: dialect parameter can be given which is used to define a set of parameters specific to a particular
        CSV dialect. Defaults to 'excel'. Other possible options are 'excel-tab' and 'unix'.
    :param delimiter: A one-character string used to separate fields. It defaults to ','.
    :param doublequote: Controls how instances of quotechar appearing inside a field should themselves be quoted.
        When True, the character is doubled. When False, the escapechar is used as a prefix to the quotechar.
        It defaults to True. On output, if doublequote is False and no escapechar is set,
        Error is raised if a quotechar is found in a field.
    :param escapechar: A one-character string used by the writer to escape the delimiter if quoting is set to
        QUOTE_NONE and the quotechar if doublequote is False. On reading, the escapechar removes any special meaning
        from the following character. It defaults to None, which disables escaping.
    :param lineterminator: The string used to terminate lines produced by the writer. It defaults to '\n'.
    :param quotechar: A one-character string used to quote fields containing special characters, such as the delimiter
        or quotechar, or which contain new-line characters. It defaults to '"'.
    :param quoting: Controls when quotes should be generated by the writer and recognised by the reader. It can take
        on any of the QUOTE_* constants (see section Module Contents) and defaults to csv.QUOTE_MINIMAL. Other options
        are csv.QUOTE_ALL, csv.QUOTE_NONNUMERIC and csv.QUOTE_NONE. Read the csv module documentation for more
        details.
    :param skipinitialspace: When True, whitespace immediately following the delimiter is ignored. The default is False.
    :param strict: When True, raise exception Error on bad CSV input. The default is False.

    :return: Path of the csv file that was written.
    """

    #A test to see if the provided data can be written into a csv file.
    try:
        x=[iter(subData) for subData in data]
        logger.debug("Iterator test passed. Looks like the data can be written into a csv file.")

    except TypeError:
        logger.error("The type of 'data' is %s. Iterator test failed. Printing trace-back" % type(data))
        raise Exception("The provided 'data' is not iterable and cannot be written to a csv file")



    with open(filePath,"w") as dataWrite:
        csvWrite=csv.writer(dataWrite,dialect=dialect,delimiter=delimiter,doublequote=doublequote,escapechar=escapechar,
                            lineterminator=lineterminator,quotechar=quotechar,quoting=quoting,skipinitialspace=skipinitialspace,
                            strict=strict)
        if dataHeaderRow:
            csvWrite.writerow(dataHeaderRow)

        csvWrite.writerows(data)

    logger.debug("Wrote the 'data' into the file %s containing%s %s rows"%(filePath," a header row and " if dataHeaderRow else "",len(data)))

    return filePath



if __name__ =="__main__":
    pass
