"""
Functions to
"""
import logging
import shutil
import os
import sys

try:
    from ._dependencies.pathsForDeleter import Directory as DirectoryDel,FileObj as FileObjDel
    from ._dependencies.send2trash import send2trash
except ImportError:
    from _dependencies.pathsForDeleter import Directory as DirectoryDel, FileObj as FileObjDel
    from _dependencies.send2trash import send2trash

logger=logging.getLogger("__main__")


def __walklevel(path, depth = 1, filterSubDirLettersInclude=None, filterSubDirLettersExclude=None,
                subDirletterSearchCaseSensitive=True, subDirfilterAppliesToEntirePath=False):
    """
    An implementation of os.walk wherein the recursion depth, ie. how deep the folder level should be 'walked' can be specified
    with the depth parameter.

    Usage:
        >>> for roots, subs, fileLists in __walklevel(r"c:\somedir",depth=2):
        >>>


    :param path: The directory path that is to be 'walked'.
    :param depth: The recursion depth that is to be followed. 1 will return the current directory, any value above 1
        will recurse to that value. -1 will go through the entire directory.
    :return: A generator object will make it possible to iterate through files and folders the same way as os.walk.
    """
    putStringInList = lambda x: [x] if isinstance(x, str) else list(
        x) if x else None

    filterSubDirLettersInclude = putStringInList(filterSubDirLettersInclude)

    filterSubDirLettersExclude = putStringInList(filterSubDirLettersExclude)

    if not subDirletterSearchCaseSensitive:
        if filterSubDirLettersExclude:
            filterSubDirLettersExclude=[val.lower() for val in filterSubDirLettersExclude]
        if filterSubDirLettersInclude:
            filterSubDirLettersInclude=[val.lower() for val in filterSubDirLettersInclude]

    # If depth is negative, just walk
    # Not using yield from for python2 compat
    # and copy dirs to keep consistant behavior for depth = -1 and depth = inf
    if depth < 0:
        #Note: I changed this to 30,000 or rather some really high number
        depth=30000
        # for root, dirs, files in os.walk(path):
        #     yield root, dirs[:], files
        # return
    elif depth == 0:
        return

    # path.count(os.path.sep) is safe because
    # - On Windows "\\" is never allowed in the name of a file or directory
    # - On UNIX "/" is never allowed in the name of a file or directory
    # - On MacOS a literal "/" is quitely translated to a ":" so it is still
    #   safe to count "/".
    base_depth = path.rstrip(os.path.sep).count(os.path.sep)
    for root, dirs, files in os.walk(path):
        #This is to check
        if subDirfilterAppliesToEntirePath:
            folderStrToCheck=root
        else:
            parent,folderStrToCheck=os.path.split(root)
            folderStrToCheck=folderStrToCheck if folderStrToCheck else parent

        folderStrToCheck=folderStrToCheck if subDirletterSearchCaseSensitive else folderStrToCheck.lower()

        if filterSubDirLettersInclude:
            for filtLet in filterSubDirLettersInclude:
                if filtLet in folderStrToCheck:
                    yield root, dirs[:], files
        elif filterSubDirLettersExclude:
            exclFound=False
            for filtLet in filterSubDirLettersExclude:
                if filtLet in folderStrToCheck:
                    exclFound=True
                    break
            if not exclFound:
                yield root, dirs[:], files
        else:
            yield root, dirs[:], files

        cur_depth = root.count(os.path.sep)
        if base_depth + depth <= cur_depth:
            del dirs[:]

def __delFileDir(fileDirPath:str, isFile:bool,silentMode:bool=True, permanent=False, tryPermIfTempFails=False) ->bool:
    """
    Delete a file or folder. This function should not be invoked directly and is only meant as a helper for other functions.

    :param fileDirPath: The file or directory path which is to be deleted.
    :param isFile: True if the path is supposed to be a file, False if it is supposed to be a directory.
    :param silentMode: If set to True, in case the deletion fails, the error will be logged, but the script wont crash.
        If set to False, then the script will crash by re-raising the exception that was issued when the file/folder was not found
        or could not be deleted.
    :param permanent: If set to True, then the file/folder will be deleted and cannot be retrieved. If set to False it will
        be sent to the equivalent of recycle bin (depending on the OS) for where it can hopefully be retrieved.
    :param tryPermIfTempFails: If permanent is set to False and the file/folder cannot be deleted temporarily (moved to recycle bin),
        then try permanent deletion.
    :return: Return a True status if delete went on smoothly.

    """
    deleteSuccessfull=False

    pathType="file" if isFile else "directory"
    delType="permanently" if permanent else "temporarily"

    try:
        #This will check if the path exists.

        logger.debug("About to delete the %s %s %s."%(pathType,fileDirPath,delType))
        if isFile:
            fileObj=FileObjDel(fileDirPath)
        else:
            direc=DirectoryDel(fileDirPath)

        if not permanent:
            try:

                send2trash(fileDirPath)
                if os.path.exists(fileDirPath):
                    logger.debug("Could not delete the %s %s temporarily by sending to trash" % (pathType,fileDirPath))
                    if tryPermIfTempFails:
                        logger.debug("Trying permanent deletion")

                        if isFile:
                            os.remove(fileDirPath)
                        else:
                            shutil.rmtree(fileDirPath)

                        deleteSuccessfull=True
                    elif not silentMode:
                        raise Exception("Raising exception as the silentMode is set to False.")

                else:
                    logger.debug("Sent the %s %s to trash" % (pathType,fileDirPath))
                    deleteSuccessfull=True
            except:
                logger.error("Could not delete the %s temporarily"%pathType)
        else:
            if isFile:
                os.remove(fileDirPath)
            else:
                shutil.rmtree(fileDirPath)

            logger.debug("Successfully deleted the %s %s" % (pathType,fileDirPath))
            deleteSuccessfull=True
    except:
        logger.error("Could not delete the  %s %s" % (pathType,fileDirPath))

        if not silentMode:
            logger.debug("About to raise exception as silentMode is set to False.")
            exType,exValue,traceback=sys.exc_info()
            raise Exception(exValue)

    return deleteSuccessfull

def deleteFile(filePath:str, silentMode:bool=True, permanent=False, tryPermIfTempFails=False) ->bool:
    """
    Delete a file.

    :param filePath: The file path which is to be deleted.
    :param silentMode: If set to True, in case the deletion fails, the error will be logged, but the script wont crash.
        If set to False, then the script will crash by re-raising the exception that was issued when the file was not found
        or could not be deleted.
    :param permanent: If set to True, then the file will be deleted and cannot be retrieved. If set to False it will
        be sent to the equivalent of recycle bin (depending on the OS) for where it can hopefully be retrieved.
    :param tryPermIfTempFails: If permanent is set to False and the file cannot be deleted temporarily (moved to recycle bin),
        then try permanent deletion.
    :return: Return a True status if delete went on smoothly.

    """
    return __delFileDir(fileDirPath=filePath,isFile=True,silentMode=silentMode,permanent=permanent,tryPermIfTempFails=tryPermIfTempFails)

def deleteDirectory(dirPath:str, silentMode:bool=True, permanent=False, tryPermIfTempFails=False) ->bool:
    """
    Delete a directory recursively with everything inside it.

    :param dirPath: The directory path which is to be deleted.
    :param silentMode: If set to True, in case the deletion fails, the error will be logged, but the script wont crash.
        If set to False, then the script will crash by re-raising the exception that was issued when the directory was not found
        or could not be deleted.
    :param permanent: If set to True, then the directory will be deleted and cannot be retrieved. If set to False it will
        be sent to the equivalent of recycle bin (depending on the OS) for where it can hopefully be retrieved.
    :param tryPermIfTempFails: If permanent is set to False and the directory cannot be deleted temporarily (moved to recycle bin),
        then try permanent deletion.
    :return: Return a True status if delete went on smoothly.

    """
    return __delFileDir(fileDirPath=dirPath, isFile=False, silentMode=silentMode, permanent=permanent, tryPermIfTempFails=tryPermIfTempFails)

#Delete files only in a directory.
# Silent mode, crash mode.
# Specify how many levels deep should it look. -1 means that it will delete files at all levels. 0 implies root level, 1 implies 1st subfolder level, 2 implies upto second subfolder level etc.
# This will essentially clean out a directory of all files.
#
def deleteDirFilesOnly(dirPath:str,delLevel=0 ,silentMode:bool=True, permanent=False, tryPermIfTempFails=False,
                       filterExtensionIncl=None, filterExtensionExcl=None,
                       filterLettersExclude=None, filterLettersInclude=None,
                       filterEndsWith=None, filterStartsWith=None,
                       letterSearchCaseSensitive=True, sortByName=True, filterExtensionsCaseSensitive=False,
                       filterSubDirLettersInclude=None, filterSubDirLettersExclude=None,
                        subDirletterSearchCaseSensitive=True,subDirfilterAppliesToEntirePath=False,
                       createdAfter=None,createdBefore=None,modifiedAfter=None,
                       modifiedBefore=None,sizeGreaterThanOrEqual=None,sizeLessThanOrEqual=None):
    """
    Delete files in a directory (and optionally its subdirectories), while leaving the directory structure intact.
    By default, only files at the root level will be deleted, however, this can be controlled by using the delLevel parameter.

    Since deletion is a risk process, the filtering here will follow a top-down approach in case of parameters such as size,
    modification date and creation date. Application of each of these filters will result in further filtering and thus
    lesser number of files being considered. If all filtering parameters are given, the order of filtering will be,
    creation date, modification date and size.


    :param dirPath: The directory path from which the files are to be deleted.
    :param silentMode: If set to True, in case the deletion fails, the error will be logged, but the script wont crash.
        If set to False, then the script will crash by re-raising the exception that was issued when the directory was not found
        or could not be deleted.
    :param permanent: If set to True, then the directory will be deleted and cannot be retrieved. If set to False it will
        be sent to the equivalent of recycle bin (depending on the OS) for where it can hopefully be retrieved.
    :param tryPermIfTempFails: If permanent is set to False and the directory cannot be deleted temporarily (moved to recycle bin),
        then try permanent deletion.
    :param filterExtensionIncl: Include these file extensions.
    :param filterExtensionExcl: Exclude these file extensions.
    :param filterLettersExclude: Look for file entries that do not contain this string.
    :param filterLettersInclude: Look for file entries that contain this string.
    :param filterEndsWith: Look for file entries that end with this string. Note that the
        extension should be included with the string. ..i.e. "oom.rad"
    :param filterStartsWith: Look for file entries that start with this string. The searching
        for this begins at the file level only and not the directory level.
    :param letterSearchCaseSensitive: If strings are being searched then setting this to
        False makes the search case insensitive.
    :param filterExtensionsCaseSensitive: If set to True then filterLettersInclude and filterLettersExclude options will
        be considered on a case-sensitive basis.
    :param filterSubDirLettersInclude:Consider only those subdirectories that match this string. Defaults to None.
    :param filterSubDirLettersExclude:Consider only those subdirectories that do note match this string. Defaults to None.
    :param subDirletterSearchCaseSensitive: Search is case sensitive. Defaults to True.
    :param createdAfter: Delete only those files that were created after this date.
    :param createdBefore: Delete only those files that were created before this date.
    :param modifiedAfter: Delete only those files that were modified after this date.
    :param modifiedBefore: Delete only those files that were modified before this date.
    :param sizeGreaterThanOrEqual: Consider for deletion if the size is greater than or equal to this in KB.
    :param sizeLessThanOrEqual: Consider for deletion if the size is less than or equal to this in KB.
    :return:
    """

    fileFilterArgsFile={"filterExtensionIncl":filterExtensionIncl,"filterExtensionExcl":filterExtensionExcl,
                    "filterLettersExclude":filterLettersExclude,"filterLettersInclude":filterLettersInclude,
                    "filterEndsWith":filterEndsWith,"filterStartsWith":filterStartsWith,
                    "letterSearchCaseSensitive":letterSearchCaseSensitive, "sortByName":sortByName,
                    "filterExtensionsCaseSensitive":filterExtensionsCaseSensitive,"createdAfter":createdAfter,
                    "createdBefore":createdBefore,"modifiedBefore":modifiedBefore,"modifiedAfter":modifiedAfter,
                        "sizeLessThanOrEqual":sizeLessThanOrEqual,"sizeGreaterThanOrEqual":sizeGreaterThanOrEqual}

    dirClass =DirectoryDel(dirPath)

    for root,subs,fileList in dirClass.walkRecurFilter(depth=delLevel,filterSubDirLettersExclude=filterSubDirLettersExclude,
                                                       filterSubDirLettersInclude=filterSubDirLettersInclude,
                                                       subDirletterSearchCaseSensitive=subDirletterSearchCaseSensitive,
                                                       subDirfilterAppliesToEntirePath=subDirfilterAppliesToEntirePath,retRootAsDirObj=True):
        for fn in root.fileNamesAbsolutePlus(**fileFilterArgsFile):
            deleteFile(fn,silentMode=silentMode,permanent=permanent,tryPermIfTempFails=tryPermIfTempFails)


#TODO : A function to delete subdirectories.This function seems to be unworkable, at least using the os.walk philosophy, at it is also probably dangerous.
# I might end up delete useful files by delete folders altogether.
# def deleteDirSubs(dirPath:str,delLevel=0 ,silentMode:bool=True, permanent=False, tryPermIfTempFails=False,
#                   filterSubDirLettersInclude=None, filterSubDirLettersExclude=None,
#                 subDirletterSearchCaseSensitive=True,subDirfilterAppliesToEntirePath=False,):




if __name__ == "__main__":
    from sarUtil.log import LoggerMain

    logger=LoggerMain(stack_infoError=True,exc_infoError=True)
    pass

