"""Contains a few functions for reading strings, normal or binary, from files. Has a whole bunch of testing, and additional
 functionlity built into it. Presently, two functions are available:

 + readText: Read a normal text file and return output as string.
 + readBinary: Read a binary file and return output.

 """

import os
import logging
import locale
import  sys

logger=logging.getLogger("__main__")

def __readerTemplate(fileName:str, mode:str,log=True, encoding:str=None, errors=None, newLine:str=None,
                     fileSizeLimitKBToDisableLog:int=1024):
    r"""
    This is the general template function for reading in different modes. This function should not be called on it's own
    and is meant to be invoked by other functions with limited options instead.

    :param string: The string that is to be written to file. Can be a byte-string or a regular string.
    :param fileName: The file path to which the string is to be written to.
    :param mode: The writing mode. Valid modes are: 'w':writing ,'a':appending ,'w+b':binary writing ,'w+':writing ,'wb':binary writing.
        If the mode is any other than these, then an exception will be raised through an assert statement.
    :param log: If set to True, then different stages of the write operation will be logged.
    :param encoding: Encoding option for writing. Since the default is None, the open function will fetch the value from
         locale.getpreferredencoding(). Other plausible options are: utf-8,utf8,latin-1,latin1,iso-8859-1,iso8859-1,
         mbcs,ascii,us-ascii,utf-16,utf16,utf-32,utf32,utf_8,latin_1,iso_8859_1,iso8859_1,us_ascii,utf_16,utf_32
    :param errors: errors is an optional string that specifies how encoding and decoding errors are to be handled—this
        cannot be used in binary mode. A variety of standard error handlers are available (listed under Error Handlers),
        though any error handling name that has been registered with codecs.register_error() is also valid.
    :param newLine: Controls how universal newlines mode works (it only applies to text mode). It can be None,
        '', '\n', '\r', and '\r\n'.
    :param fileSizeLimitKBToDisableLog: If the size of the file is beyond this value in KiloBytes, then disable
        logging operations that require reading into the string. This is being done to prevent unncessary bottle necks.

    :return: The string or byte-string that was read.
    """

    logger.debug("Trying to read the file: %s"%fileName)

    assert os.path.exists(fileName),"The file %s does not exist and therefore cannot be read."%fileName

    fileSize=os.path.getsize(fileName)/1024

    disableReadLogging=True if (fileSize>fileSizeLimitKBToDisableLog or  "b" in mode) else False

    prefEncoding=locale.getpreferredencoding()

    otherEncodings=["utf-8", "utf8", "latin-1", "latin1", "iso-8859-1", "iso8859-1", "mbcs", "ascii",
                    "us-ascii", "utf-16", "utf16", "utf-32", "utf32",'utf_8', 'latin_1', 'iso_8859_1',
                    'iso8859_1', 'us_ascii', 'utf_16', 'utf_32']


    possibleEncodings=[prefEncoding]+otherEncodings

    possibleModes={"r":"reading","rt":"reading","r+":"reading","r+b":"binary reading","rb":"binary reading"}

    assert mode in possibleModes,"The specified mode '%s' is not valid. Only the following types are valid: %s"%(mode,",".join(["'%s':%s "%(key,val) for key,val in possibleModes.items()]))


    if log:

        if encoding and not (encoding in possibleEncodings):
            logger.warning("The specified encoding type '%s' is not within the possible list of encodings. "
                           "The write operation might not be successful"%encoding)

    if 'b' in mode:
        assert encoding is None,"No encoding should be specified for binary mode."
        assert newLine is None,"No newline argument (%s) is allowed for writing in binary mode"%newLine
        fileToRead=open(fileName,mode,errors=errors,newline=newLine,)
    else:

        fileToRead = open(fileName, mode, encoding=encoding, errors=errors, newline=newLine)

    output=None
    with fileToRead:


        try:
            output =fileToRead.read()

        except UnicodeDecodeError:
            msg="\n\nIT IS MOST LIKELY THAT A NORMAL TEXT-READ OPERATION WAS ATTEMPTED ON BINARY FILE \n"*2
            logger.error("A UnicodeDecode exception occured due to either an incorrectly opened file or incorrectly specified encoding type: '%s'%s\n\n"%(encoding,msg))

            k=sys.exc_info()
            raise Exception(sys.exc_info()[0])

        if log:
            mode="mode: '%s'(%s)"%(fileToRead.mode,possibleModes[fileToRead.mode])
            try:
                encoding,errors,newLine=", encoding: '%s',"%fileToRead.encoding,"errors: '%s', "%fileToRead.errors,\
                                        "newLines: '%s'."%(list(fileToRead.newlines or newLine))
            except AttributeError:
                encoding,errors,newLine="","",""
            except TypeError:
                encoding, errors, newLine = ", encoding: '%s',"%fileToRead.encoding, "", ""
                logger.warning("It seems that a text-read operation was specified for a binary encoded file. The output will be returned as a string.")

            logger.debug("The write operation used following attributes: %s%s%s%s"%(mode,encoding,errors,newLine))

    logger.debug("Finished read, the 'closed' status of the file descriptor is '%s'." % fileToRead.closed)

    if log:
        if (output is None):
            logger.debug("It is likely that something went wrong in the read operation as no output was recorded.")
            logger.debug("Returning None and exiting")
        else:
            if (not disableReadLogging) :
                outputType,outputLen=type(output),len(output)
                logger.debug("Read a string of type %s and of length %s"%(outputType,outputLen))

                if "\n" in output:
                    logger.debug("The output contains %s line-breaks"%(output.count("\n")))
            else:
                logger.debug("Not counting line breaks and determining type of the string as either this was a byte-read operation or the size: "
                             "(%0.3f)KB is greater than the limit (%s)KB specified through 'fileSizeLimitKBToDisableLog'"%(fileSize,fileSizeLimitKBToDisableLog))
    return output

def readText(fileName:str, log=True, encoding:str=None, errors=None, newLine:str=None,fileSizeLimitKBToDisableLog:int=1024) ->str:
    r"""
    Read a normal text file and return output as a string.

    :param fileName: The file path to which the string is to be written to.
    :param log: If set to True, then different stages of the write operation will be logged.
    :param encoding: Encoding option for writing. Since the default is None, the open function will fetch the value from
         locale.getpreferredencoding(). Other plausible options are: utf-8,utf8,latin-1,latin1,iso-8859-1,iso8859-1,
         mbcs,ascii,us-ascii,utf-16,utf16,utf-32,utf32,utf_8,latin_1,iso_8859_1,iso8859_1,us_ascii,utf_16,utf_32
    :param errors: errors is an optional string that specifies how encoding and decoding errors are to be handled—this
        cannot be used in binary mode. A variety of standard error handlers are available (listed under Error Handlers),
        though any error handling name that has been registered with codecs.register_error() is also valid.
    :param newLine: Controls how universal newlines mode works (it only applies to text mode). It can be None,
        '', '\n', '\r', and '\r\n'.
    :param fileSizeLimitKBToDisableLog: If the size of the file is beyond this value in KiloBytes, then disable
        logging operations that require reading into the string. This is being done to prevent unncessary bottle necks.

    :return: The string that was read.
    """
    return __readerTemplate(fileName=fileName,mode="r",log=log,encoding=encoding,errors=errors,newLine=newLine,
                            fileSizeLimitKBToDisableLog=fileSizeLimitKBToDisableLog)

def readBinary(fileName:str, log=True, fileSizeLimitKBToDisableLog:int=1024) ->bytes:
    r"""
    Read a binary file and return output.

    :param fileName: The file path to which the string is to be written to.
    :param log: If set to True, then different stages of the write operation will be logged.
    :param encoding: Encoding option for writing. Since the default is None, the open function will fetch the value from
         locale.getpreferredencoding(). Other plausible options are: utf-8,utf8,latin-1,latin1,iso-8859-1,iso8859-1,
         mbcs,ascii,us-ascii,utf-16,utf16,utf-32,utf32,utf_8,latin_1,iso_8859_1,iso8859_1,us_ascii,utf_16,utf_32
    :param errors: errors is an optional string that specifies how encoding and decoding errors are to be handled—this
        cannot be used in binary mode. A variety of standard error handlers are available (listed under Error Handlers),
        though any error handling name that has been registered with codecs.register_error() is also valid.
    :param newLine: Controls how universal newlines mode works (it only applies to text mode). It can be None,
        '', '\n', '\r', and '\r\n'.
    :param fileSizeLimitKBToDisableLog: If the size of the file is beyond this value in KiloBytes, then disable
        logging operations that require reading into the string. This is being done to prevent unncessary bottle necks.

    :return: The byte string that was read.
    """

    return __readerTemplate(fileName=fileName,mode="rb",log=log,fileSizeLimitKBToDisableLog=fileSizeLimitKBToDisableLog)


if __name__ == "__main__":
    pass