"""
Date: 24 August 2018

Contains a class called LoggerMain that is a all-in-one solution for logging. See usage.
"""
import logging


class LoggerMain(logging.Logger):
    """
    This class was written to allow a one line implementation of a logging session. At its core, it inherits from
    logging.Logger and then copies the entire dict from logging.getLogger("__main__").

    It is meant to be used in the main script when I am playing around with other scripts.

    Simple usage is something like this:
        from loggerMain import LoggerMain

        logger=LoggerMain() #And thats all!!

        #Then subsequent calls can be made just like a normal logger.
        logger.debug("What?!")

    Customizing the logger with options from the StringFormats class.

        from loggerMain import LoggerMain

        from fmtStrings import StringFormats

        logger=LoggerMain(format=StringFormats.timeLvlModlFuncMsg)


    A screen logger is assigned by default. File handler can be assigned if needed. Similarly formatting is assigned
    as well.
        """
    __initialized=False
    "This attribute will be set to True once a single instance of this class has been instantiated. For a given session," \
    "only instance should be used."

    def __init__(self, name="__main__",logFile=None,logToScreen=True,format='%(asctime)s "%(levelname)s"|mod:%(module)s|line:%(lineno)d|func:%(funcName)s|msg:%(message)s',
                 timeFormat="%H:%M:%S",level=logging.DEBUG,logFileMode="a",mainCheck=True,exc_infoError=True,
                 stack_infoError=True,exc_infoInfo=False,stack_infoInfo=False,exc_infoDebug=False,stack_infoDebug=False,
                 exc_infoWarning=False,stack_infoWarning=False,exc_infoCritical=True,stack_infoCritical=True,
                 exc_infoLog=False,stack_infoLog=False,exc_infoException=True,stack_infoException=True):
        """
        Initialize a logger with the following parameters. DO NOT change "__main__" !

        :param name: Name of the logger. Do not change this one.!!! Using "__main__"  across all scripts allows me
            to use the same settings across all of my scripts.
        :param logFile: Name of a filepath to log to. Defaults to None, ie no logging to a file.
        :param logToScreen: Defaults to True, ie will log to a screen. It might useful to turn this off in situations
            where I am trying to log but also get pretty print outputs.
        :param format: Format string for formatter. Defaults to '%(asctime)s |md:%(module)s |fun%(funcName)s |msg%(message)s'.
            Note that this can also be set using the StringFormats container class in fmtStrings module.
        :param timeFormat: Time format for formatter . Defaults to hour minute second. "%H:%M:%S". Note that this
             can also be set using the TimeFormats container class in fmtStringsTime module.
        :param level: Logging level. Defaults to DEBUG.
        :param logFileMode: Defaults to a (for append).
        :param mainCheck: Throw a warning in case this class is being instantiated from a non main script. This class should
            not be instantiated trough a non main script as there would be no control over the parameters of the main
            logger then.
        :param exc_infoError: Include exc_info for the logging level error.
        :param stack_infoError: Include stack_info for the logging level error.
        :param exc_infoInfo: Include exc_info for the logging level info.
        :param stack_infoInfo: Include stack_info for the logging level info.
        :param exc_infoDebug: Include exc_info for the logging level debug.
        :param stack_infoDebug: Include stack_info for the logging level debug.
        :param exc_infoWarning: Include exc_info for the logging level warning.
        :param stack_infoWarning: Include stack_info for the logging level warning.
        :param exc_infoCritical: Include exc_info for the logging level critical.
        :param stack_infoCritical: Include stack_info for the logging level critical.
        :param exc_infoLog: Include exc_info for the general logging 'log' function.
        :param stack_infoLog: Include stack_info for the logging 'log' function.
        :param exc_infoException: Include exc_info for the logging level error.
        :param stack_infoException: Include stack_info for the logging level error.
        """


        #24 August 2018: Not sure if this assigning dicts is safe, but it seems to be working (for now!)
        logging.Logger.__init__(self,name)
        self.__dict__=logging.getLogger(name).__dict__
        formatter=logging.Formatter(format,timeFormat)

        self.setLevel(level)

        if logToScreen:
            self.defaultScreenLogger=logging.StreamHandler()
            self.defaultScreenLogger.setFormatter(formatter)
            self.addHandler(self.defaultScreenLogger)
        else:
            self.defaultScreenLogger=None

        if logFile:
            self.defaultFileLogger = logging.FileHandler(logFile,mode=logFileMode)
            self.defaultFileLogger.setFormatter(formatter)
            self.addHandler(self.defaultFileLogger)
        else:
            self.defaultScreenLogger = None

        if mainCheck:
            if LoggerMain.__initialized:
                import warnings
                warnings.warn("\n\tThe '__main__' logger has already been initiated once. Initiating again is a huge PITA!!\n"*3)
            else:
                LoggerMain.__initialized=True


        self.exc_infoError=exc_infoError
        """Change this attribute to True or False to include the exc_info for the messages logger with the error() logging
        function"""

        self.stack_infoError=stack_infoError
        """Change this attribute to True or False to include the stack_info for the messages logger with the error() logging
                function"""

        self.exc_infoInfo=exc_infoInfo
        """Change this attribute to True or False to include the exc_info for the messages logger with the info() logging
                function"""

        self.stack_infoInfo=stack_infoInfo
        """Change this attribute to True or False to include the stack_info for the messages logger with the info() logging
                function"""

        self.exc_infoDebug=exc_infoDebug
        """Change this attribute to True or False to include the exc_info for the messages logger with the debug() logging
                function"""

        self.stack_infoDebug=stack_infoDebug
        """Change this attribute to True or False to include the stack_info for the messages logger with the debug() logging
                function"""

        self.exc_infoWarning=exc_infoWarning
        """Change this attribute to True or False to include the exc_info for the messages logger with the warning() logging
                function"""

        self.stack_infoWarning=stack_infoWarning
        """Change this attribute to True or False to include the stack_info for the messages logger with the warning() logging
                function"""

        self.exc_infoCritical=exc_infoCritical
        """Change this attribute to True or False to include the exc_info for the messages logger with the critical() logging
                function"""

        self.stack_infoCritical=stack_infoCritical
        """Change this attribute to True or False to include the stack_info for the messages logger with the critical() logging
                function"""

        self.exc_infoLog=exc_infoLog
        """Change this attribute to True or False to include the exc_info for the messages logger with the log() logging
                function"""

        self.stack_infoLog=stack_infoLog
        """Change this attribute to True or False to include the stack_info for the messages logger with the log() logging
                function"""

        self.exc_infoException=exc_infoException
        """Change this attribute to True or False to include the exc_info for the messages logger with the exception() logging
                function"""

        self.stack_infoException=stack_infoException
        """Change this attribute to True or False to include the stack_info for the messages logger with the exception() logging
                function"""

    def setLevel_DEBUG(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.DEBUG)

    def setLevel_WARNING(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.WARNING)

    def setLevel_ERROR(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.ERROR)

    def setLevel_INFO(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.INFO)

    def setLevel_CRITICAL(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.CRITICAL)

    def setLevel_NOTSET(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.NOTSET)

    def info(self,msg):
        """
        :param msg: The message that is to be logged.
        """
        super().info(msg,exc_info=self.exc_infoInfo,stack_info=self.stack_infoInfo)

    def debug(self,msg):
        """
        :param msg: The message that is to be logged.
        """
        super().debug(msg,exc_info=self.exc_infoDebug,stack_info=self.stack_infoDebug)

    def warning(self,msg):
        """
        :param msg: The message that is to be logged.
        """
        super().warning(msg,exc_info=self.exc_infoWarning,stack_info=self.stack_infoWarning)

    def critical(self,msg):
        """
        :param msg: The message that is to be logged.

        """
        super().critical(msg,exc_info=self.exc_infoCritical,stack_info=self.stack_infoCritical)

    def error(self,msg):
        """
        :param msg: The message that is to be logged.

        """
        super().error(msg,exc_info=self.exc_infoError,stack_info=self.stack_infoError)

    def log(self,level,msg):
        """

        :param level: This is the level for which the logging will be done.
        :param msg: The message that is to be logged.
        :param exc_info: Include the sys.exc_info message in the log.
        :param stack_info: Include the full stack_info in case of exceptions.
        """
        super().log(level,msg,exc_info=self.exc_infoLog,stack_info=self.stack_infoLog)

    def exception(self,msg):
        """
        I am setting the values of exc_info and stack_info true so that there is a better
        idea of how and where the error occured, as and when it did.

        :param msg: The message that is to be logged.
        :param exc_info: Include the sys.exc_info message in the log.
        :param stack_info: Include the full stack_info in case of exceptions.
        """
        super().exception(msg,exc_info=self.exc_infoException,stack_info=self.stack_infoException)

if __name__ =="__main__":
    pass