"""
Multi-threading using multiprocessing.Pool mappers. The following functions are available:
 poolMapper:  Take any function 'func' and run it on multiple threads for iterating through the variable given in 'varList'

As of 28 April 2021, Pool.map and Pool.starmap have been implemented.

TODO: Allow for scenarios where the process will continue even if certain instances will fail. Write now one exception
    will crash everything.


"""

import multiprocessing as mp
import functools
import datetime
import logging
import time

logger=logging.getLogger("__main__")



def poolMapper(func,varList:list,numThreads:int,**kwargs):
    """
    Take any function 'func' and run it on multiple threads for iterating through the variable given in 'varList'.

    CRITICAL!!: This function can only be invoked after if __name__ == "__main__" due to some pickling nonsense. So,
    if on the same module, the function should defined above if __name__ == "__main__": and then called after  if __name__ == "__main__":

    Usage:
        for the function defined as following:
            def actualFunc(a,b,c,d):
                return a+b+c+d

        For 15 values of a, while keeping b,c,d constant, this function can then be paralleized as:
            if __name__ == "__main__":
                poolMapper(actualFunc,range(15),5,b=3,c=10,d=15)

    This is likely to come in handy for making os.system calls and such. For example, while downloading a bunch of stuff
    using youtube-dl, I can use this by passing the list of videoUrls as the varList while everything else remains the
    same.

    :param func: This is the function that is to be called. Do not call the function within the parameter! See the example
        above.
    :param varList: This is variable that will change.
    :param numThreads: Number of threads.
    :param kwargs: These should be the positional variable of the function func.
    :return:
    """

    logger.debug("About to parallelize %s iterations of the function %s with %s threads"%(len(varList),func.__name__,numThreads))

    fixedArgsStr=["%s=%s"%(key,val) for key,val in kwargs.items()]

    logger.debug("The following variables are fixed:\n\t%s"%("\n\t".join(fixedArgsStr)))
    logger.debug("The above variables will be encapsulated into a partial function using functools.partial")

    newFunc= functools.partial(func,**kwargs)

    start=datetime.datetime.now()


    with mp.Pool(numThreads) as proc:
        logger.debug("Starting the pool process.")
        calculatedValues = proc.map(newFunc,varList)

    end=datetime.datetime.now()

    timeInMs=int((end-start).total_seconds()*1000)

    logger.debug("Completed. %s iterations completed in %s milliseconds."%(len(calculatedValues),timeInMs))
    logger.debug('Returning the individual return values of the function "%s" as a list of %s values'%(func.__name__,len(calculatedValues)))

    return calculatedValues


def poolStarMapper(func,varLists,numThreads,**kwargs):
    """
        Take any function 'func' and run it on multiple threads for iterating through the variable given in 'varList'.

    CRITICAL!!: This function can only be invoked after if __name__ == "__main__" due to some pickling nonsense. So,
    if on the same module, the function should defined above if __name__ == "__main__": and then called after  if __name__ == "__main__":

    Usage:
        for the function defined as following:
            def actualFunc(a,b,c,d):
                return a+b+c+d

        For 4 values of (a,b) while keeping c,d constant, this function can then be paralleized as:
            if __name__ == "__main__":
                poolStarMapper(actualFunc,((1,2),(3,4),(5,6),(7,8)),5,c=10,d=15)

    :param func: This is the function that is to be called. Do not call the function within the parameter! See the example
        above.
    :param varLists: This is variable that will change. This will be a list of lists or something like that. See the
        example above.
    :param numThreads: Number of threads.
    :param kwargs: These should be the positional variable of the function func.
    :return:
    """

    varLists=list(varLists)

    logger.debug("About to parallelize %s iterations of the function %s with %s threads" % (
    len(varLists), func.__name__, numThreads))

    firstArgListLen=len(varLists[0])

    logger.debug("The number of parameters for the first iteration, and hopefully subsequent iterations is %s"%firstArgListLen)

    allArgListLen=sorted(set([len(argList) for argList in varLists]))
    assert len(allArgListLen)==1,\
        "The number of parameters for each iteration should be equal to %s. However, the following combinations of parameters were found: %s"%(firstArgListLen,",".join(map(str,allArgListLen)))

    fixedArgsStr = ["%s=%s" % (key, val) for key, val in kwargs.items()]

    logger.debug("The following variables are fixed:\n\t%s" % ("\n\t".join(fixedArgsStr)))
    logger.debug("The above variables will be encapsulated into a partial function using functools.partial")

    newFunc = functools.partial(func, **kwargs)

    start = datetime.datetime.now()

    with mp.Pool(numThreads) as proc:
        logger.debug("Starting the pool process.")
        calculatedValues = proc.starmap(newFunc, varLists)

    end = datetime.datetime.now()

    timeInMs = int((end - start).total_seconds() * 1000)

    logger.debug("Completed. %s iterations completed in %s milliseconds." % (len(calculatedValues), timeInMs))
    logger.debug('Returning the individual return values of the function "%s" as a list of %s values' % (
    func.__name__, len(calculatedValues)))

    return calculatedValues




def __actualFunc(a, b, c, d):
    """
    This is just a test function for checking the other functions. Since it starts with a __ it wont get mistaken
    for something useful outside the module.
    :param a:
    :param b:
    :param c:
    :param d:
    :return:
    """
    print("Calling %s" % a)
    time.sleep(4)
    return a + b + c + d

if __name__ == "__main__":
    from sarUtil.log import LoggerMain

    logger = LoggerMain()

    print(poolStarMapper(__actualFunc,((1,2),(3,4),(5,9),(6,7),(8,9)),numThreads=11,c=10,d=15))