"""
Utility functions for string-related operations.
"""
import logging

logger=logging.getLogger("__main__")

__uniCodeDictionary = {'\xe2\x88\x92': '-',
                     '\xe2\x80\x99':"'",
                     '\xe2\x80\x98':"'",
                     '\xcb\x9c':"~",
                     '\xcb\x86':"^",
                     '\xc2\xb0':" degrees",
                     '\xc2\xb1':"+/-",
                     '\xc2\xb2':"^2",
                     '\xc2\xb3':'^3'}

def __checkInputStrForFiltering(stringList,subStrLstTup,func,inclusionTest=True):
    """
    Just a utility function to clean up inputs for the other filtering functions. This
    function is not to be accessed outside of this module.

    :param stringList: List of strings that are to be filtered.
    :param subStrLstTup: List of substrings that will form the filtering criteria.
    :param func: This will either be 'any' or 'all'
    :param inclusionTest: If set to True, then filtering will be based on whether the
        substring is present in string. If false, then it will be based it NOT being present.
    :return:
    """
    logger.debug("stringList is '%s' and subStrLstTup is '%s'"%(type(stringList),type(subStrLstTup)))

    if type(stringList) == str:
        stringList=[stringList]
        logger.debug("Wrapped the stringList, which was a string, inside a list")

    if type(subStrLstTup)==str:
        subStrLstTup=[subStrLstTup]
        logger.debug("Wrapped the subStrLstTup, which was a string, inside a list")

    logger.debug("The input list for filtering has %s values"%(len(stringList)))
    logger.debug("The list of substrings has %s values."%(len(subStrLstTup)))


    outputStringList=[]
    for string in stringList:
        if inclusionTest:
            if func([val in string for val in subStrLstTup]):
                outputStringList.append(string)
        else:
            if not func([val in string for val in subStrLstTup]):
                outputStringList.append(string)

    logger.debug("After filtering the output list contains %s values"%len(outputStringList))

    return outputStringList

def resizeString(strings,lineSize:int=80,expectedHeadingLen = 30,checkForFullStops=True):
    """
    This is mostly a utility function for generating docstrings and such for python code.
    The basic idea is to add line-breaks to long string.

    Example:
        >>> x=resizeString("I live in Bavaria, more precisely in Munich. And here there is a tradition that you may know - beer gardens. A typical beer garden has orange benches and tables. The bottom is covered with small pebbles. Large chestnut trees provide")
        >>> print(x) #I live in Bavaria, more precisely in Munich. And here there is a tradition that (a linebreak happens here)


    :param strings: Can be any string that needs to be formmated. Best option would be to write that in scrap.txt and read from there.
    :param lineSize: The line character width at which the line needs to be truncated.
    :param expectedHeadingLen: Some lines can just be headings like "-ab". So any line below this len will be classified as a heading.
    :param checkForFullStops: Boolean value to check if a heading is a line or a part of a sentence.
    :return: The formatted string as per the options.
    """

    if isinstance(strings,str):
        strings = [strings]

    for string in strings:
        for keys in __uniCodeDictionary:
            string = string.replace(keys, __uniCodeDictionary[keys])
        stringVal = string.strip().split('\n')

        stringVal = [val.strip() for val in stringVal if not val.startswith('\t')
                    and len(val)>0  and not len(val.strip())/len(val) <0.4]

        stringVal = [sentence.split() for sentence in stringVal]
        outputString = []
        tempString = ''

        for idx,sentence in enumerate(stringVal):
            if (len(" ".join(sentence)) < expectedHeadingLen) and (checkForFullStops is True) \
                    and not " ".join(sentence).endswith("."):
                tempString += "\n\n%s\n"%(" ".join(sentence))
                outputString.append(tempString)
                tempString = ''
            else:
                if not sentence and idx != (len(stringVal)-1):
                    outputString.append(tempString)
                    if tempString:
                        outputString.append('')
                    tempString = ''
                else:
                    for words in sentence:
                        if len(tempString)+len(words)<=lineSize:
                            if tempString:
                                tempString += " %s"%words
                            else:
                                tempString += words
                        else:
                            outputString.append(tempString)
                            tempString = words

        outputString.append(tempString)

        return "\n".join(outputString).strip()

def printStringIndented(stringVal:str,indent:str,stringPrefix:str="",stringSuffix:str=""):
    """
    As the title suggests, given a string, print it on screen with indentations, useful for generating comments and python code.

    Example:
        >>> nameList=['cunane', 'cunegonda', 'cunegunda']
        >>> for val in nameList:
        >>>    printStringIndented(val,indent="\t",stringPrefix="Mr. ")

    :param stringVal: The string to be indented. If it is a multiline string then indent each line.
    :param indent: Indent type. Can be tabs space etc.
    :return: None

    """
    stringList=stringVal.split("\n")

    for idx,strng in enumerate(stringList):
        if not idx and len(stringList)==1:
            print("%s%s%s%s"%(indent,stringPrefix,strng,stringSuffix))
        elif not idx and len(stringList)>1:
            print("%s%s%s" % (indent, stringPrefix, strng))
        elif idx==len(stringList)-1:
            print("%s%s%s" % (indent, strng,stringSuffix))
        else:
            print("%s%s" % (indent, strng))

def filtIncldAllSubStrings(stringList,subStrLstTup):
    """
    Given a list/tuple of strings, return only those strings that contain all the substrings given in subStrLstTup.

    Example:
        >>> nameList=['cunane', 'cunegonda', 'cunegunda', 'cunera', 'cunha', 'cunigan', 'cunningham', 'cupani', 'cupar']
        >>> k=filtIncldAllSubStrings(nameList,["cu","nd"])
        >>> k # returns ['cunegonda', 'cunegunda']

    :param stringList: This is the list/tuple of strings which needs to be filtered.
    :param subStrLstTup: This is the list or tuple of subStrings.
    :return: A list containing the strings that matched the filtering criteria.
    """
    return __checkInputStrForFiltering(stringList,subStrLstTup,all)

def filtIncldAnySubStrings(stringList,subStrLstTup):
    """
    Given a list/tuple of strings, return only those strings that contain any of the substrings
        given in subStrLstTup.

        Example:

            >>> nameList=['cunane', 'cunegonda', 'cunegunda', 'cunera', 'cunha', 'cunigan', 'cunningham', 'cupani', 'cupar']
            >>> k=filtIncldAnySubStrings(nameList,["o","p","ra"])
            >>> ['cunegonda', 'cunera', 'cupani', 'cupar']

    :param stringList: This is the list/tuple of strings which needs to be filtered.
    :param subStrLstTup: This is the list or tuple of subStrings.
    :return: A list containing the strings that matched the filtering criteria.
    """
    return __checkInputStrForFiltering(stringList, subStrLstTup, any)

def filtExcldAllSubStrings(stringList,subStrLstTup):
    """
    Given a list/tuple of strings, return only those strings that does not contain
    all the substrings given in subStrLstTup. i.e. if all the substrings are present in
    the string, then filter it out.

    Example:
        >>> nameList=['cunane', 'cunegonda', 'cunegunda', 'cunera', 'cunha', 'cunigan', 'cunningham', 'cupani', 'cupar']
        >>> k=filtExcldAllSubStrings(nameList,["cu","nd"])
        >>> k #['cunane', 'cunera', 'cunha', 'cunigan', 'cunningham', 'cupani', 'cupar']

    :param stringList: This is the list/tuple of strings which needs to be filtered.
    :param subStrLstTup: This is the list or tuple of subStrings.
    :return: A list containing the strings that matched the filtering criteria.
    """
    return __checkInputStrForFiltering(stringList,subStrLstTup,all,inclusionTest=False)

def filtExcldAnySubStrings(stringList,subStrLstTup):
    """
    Given a list/tuple of strings, return only those strings that contain any of the substrings given in subStrLstTup.i.e.
    if any of  the substrings are present in the string, then filter it out.

     Example:
        >>> nameList=['cunane', 'cunegonda', 'cunegunda', 'cunera', 'cunha', 'cunigan', 'cunningham', 'cupani', 'cupar']
        >>> k=filtExcldAnySubStrings(nameList,["nd","ne"])
        >>> k
        ['cunha', 'cunigan', 'cunningham', 'cupani', 'cupar']

    :param stringList: This is the list/tuple of strings which needs to be filtered.
    :param subStrLstTup: This is the list or tuple of subStrings.
    :return: A list containing the strings that matched the filtering criteria.
    """

    return __checkInputStrForFiltering(stringList,subStrLstTup,any,inclusionTest=False)


if __name__=="__main__":
    pass

