# coding=utf-8
"""
Contains a class that can help in creating time strings. Read the documentation for TimeSymbols
"""




class TimeSymbols(object):
    """
    This is a helper class that can be used to prototype datetime strftime formatting options. See the 'primary use' example
    below. It must not be instantiated. An error will be raised on instantiation. The idea is to simply call the attributes
    so that strings can be returned.

    **Primary Use**: This class can be used for testing and prototyping to create the right kind of formats. The primary rationale
    for coding this is that the %M etc. symbols are hard to remember. So, a way out is something like this:
        Step 1: Prototyping, figure out what symbols are needed. Autocomplete in Pycharm etc is obviously helpful here:
            >>> import TimeSymbols as ts #Note: Don't do this if this script is being tested on the same page where TimeSymbols was defined.
            >>> ts=TimeSymbols
            >>> strFmt = "%s:%s%s-%s/%s/%s" % (ts.Hour12,ts.MinuteNum,ts.AMPM,ts.DayMonth,ts.MonthNum,ts.YearWiCent)
            >>> print(strFmt) #Prints %I:%M%p-%d/%m/%Y


        Step2: Now use the returned value from previous step ie (%I:%M%p-%d/%m/%Y) to actually format the time as a string.
            >>> import datetime
            >>> currTimeStr = datetime.datetime.now().strftime("%I:%M%p-%d/%m/%Y")
            >>> print(currTimeStr) #Prints 4:21PM-18/04/2021




    **Secondary Use**:This class can be used independently as well. Although this is not recommended as the code looks pretty ugly:
    Example 1:
            >>> import datetime
            >>> ts=_TimeSymbols
            >>> print(datetime.datetime.now().strftime("%s-%s|%s:%s"%(ts.DayMonth,ts.MonthFull,ts.Hour24,ts.MinuteNum))) #Will print current Date-Month|Time: 18-April|15:59

    Example 2:
            >>> import datetime
            >>> ts=_TimeSymbols
            >>> print(datetime.datetime.now().strftime("%s%s%s%s%s%s%s"%(ts.DayMonth,ts.SepDash,ts.MonthFull,ts.SepVerticalPipeBarLine,ts.Hour24,ts.SepColon,ts.MinuteNum)))
    """
    WkDayAbbr = "%a"
    """
    Weekday as locale’s abbreviated name.

    Sun, Mon, …, Sat (en_US);
    So, Mo, …, Sa (de_DE)
    """

    WkDayFull = "%A"
    """Weekday as locale’s full name.

    Sunday, Monday, …, Saturday (en_US);
    Sonntag, Montag, …, Samstag (de_DE)
    """

    WkDay = "%w"
    """
    Weekday as a decimal number, where 0 is Sunday and 6 is Saturday.

    0, 1, …, 6"""



    DayMonth = "%d"
    """Day of the month as a zero-padded decimal number.


    01, 02, …, 31"""


    MonthAbbr = "%b"
    """Month as locale’s abbreviated name.

        Jan, Feb, …, Dec (en_US);
        Jan, Feb, …, Dez (de_DE)

    """

    MonthFull = "%B"
    """Month as locale’s full name.

    January, February, …, December (en_US);
    Januar, Februar, …, Dezember (de_DE)
    """

    MonthNum = "%m"
    """
    Month as a zero-padded decimal number.
    01, 02, …, 12"""


    YearWoCent = "%y"
    """
    Year without century as a zero-padded decimal number.
    00, 01, …, 99"""

    YearWiCent = "%Y"
    """Year with century as a decimal number.
    0001, 0002, …, 2013, 2014, …, 9998, 9999"""


    Hour24 = "%H"
    """Hour (24-hour clock) as a zero-padded decimal number.

    00, 01, …, 23"""

    Hour12 = "%I"
    """
    Hour (12-hour clock) as a zero-padded decimal number.
    01, 02, …, 12"""

    AMPM = "%p"
    """
    Locale’s equivalent of either AM or PM.

    AM, PM (en_US);
    am, pm (de_DE)
    """

    MinuteNum = "%M"
    """
    Minute as a zero-padded decimal number.

    00, 01, …, 59"""

    SecondNum = "%S"
    """
    Second as a zero-padded decimal number.

    00, 01, …, 59"""

    MicroSec="%f"
    """
    Microsecond as a decimal number, zero-padded on the left.

    000000, 000001, …, 999999
    """

    UTCoffset="%z"
    """
    UTC offset in the form ±HHMM[SS[.ffffff]] (empty string if the object is naive).

    (empty), +0000, -0400, +1030, +063415, -030712.345216
    """


    TimeZone = "%Z"
    """Time zone name (empty string if the object is naive).

    (empty), UTC, GMT"""

    DayOfYearJulian="%j"
    """
    Day of the year as a zero-padded decimal number.

    001, 002, …, 366
    """

    WkNumYearSun = "%U"
    """
    Week number of the year (Sunday as the first day of the week) as a zero padded decimal number.
    All days in a new year preceding the first Sunday are considered to be in week 0.

    00, 01, …, 53"""


    WkNumYearMon = "%W"
    """
    Week number of the year (Monday as the first day of the week) as a decimal number. All days in a new year preceding
    the first Monday are considered to be in week 0.

    00, 01, …, 53"""

    DateTimeRep = "%c"
    """Locale’s appropriate date and time representation.

    Tue Aug 16 21:30:00 1988 (en_US);
    Di 16 Aug 21:30:00 1988 (de_DE)
	"""



    DateRep = "%x"
    """Locale’s appropriate date representation.

    08/16/88 (None);
    08/16/1988 (en_US);
    16.08.1988 (de_DE)
    """

    TimeRep = "%X"
    """Locale’s appropriate time representation.

    21:30:00 (en_US);
    21:30:00 (de_DE)
    """

    SepPercent="%%"
    """
    A literal '%' character.

    %
    """

    SepSpace = " "
    """Space separator"""

    SepDash = "-"
    """Dash separator"""

    SepUnderScore = "_"
    """Underscore separator"""

    SepTab = "\t"
    """Tab separator"""

    SepLine = "\n"
    """Line separator"""

    SepColon = ":"
    """Colon separator"""

    SepBackSlash = "\\"
    """Backslash separator"""

    SepForwardSlash = "/"
    """Forwardslash separator"""

    SepVerticalPipeBarLine="|"
    """
    Vertical line or vertical bar or pipe.
    """

    def __init__(self):
        raise Exception("This class should not be instantiated. The usage is"
                        "to simply call attributes for creating time strings in "
                        "other classes")




if __name__ =="__main__":
    pass