"""
Contains a class called TimeStamp that can be used to generate timestamp strings, which can then further be used to name
files, provide labels to plots etc.
"""
try:
    from parsedatetime import DateTimeParse
except ImportError:
    from .parsedatetime import DateTimeParse

class TimeStamp(object):
    """
    A class that can be used to quickly create timestamps for inclusion in strings, appending to filenames, providing
    labels to plots etc. After instantiation, everything in this class is to be accessed using properties only.
    Accessible properties are of two types. The first are templates, and start with tmpl. These provided readymade templates
    for hour-min, MonthDateHourMin etc. The oter properties are single values relating to a specific variable of the time
    such as wkDayAbbr (for weekday abbreviated), hour24 (for the 24-hour cycle hour) and so on.

    Usage:
        Assuming it is April 29th 15:39 hours:

        >>> ts=TimeStamp(timeStrForStamp="now",separator=":")
        >>> ts.tmplMonthAbbrDateHourMin #Apr:29:15:39
        >>> ts.tmplHourMin #15:39
        >>> ts.wkDayAbbr #Thu
        >>> ts.customFormat('%H-%M') # There is one custom method that can accept an appropriate format: 15-39
        >>> '%s-%s'%(ts.wkDayAbbr,ts.tmplHourMin) #Since everything is a string, properties can be added together: Thu-15:39
    """

    def __init__(self,timeStrForStamp="now",separator="-"):
        """
        Read the class docstring for details about usage.
        :param timeStrForStamp: This will be a string that specifies the point in time for which the timestamps are to
            be generated. This string will be parsed using the parsedatetime method. Defaults to 'now', but can
            be anything like '3 days ago', '50 minutes ago' etc.

        :param separator: This is the separator that will be used to separate values in the properties starting with
        tmpl such as tmplHourMin. Defaults to '-', so tmplHourMin by default will be something like 22-11 for 10:11PM.

        """
        self.timeStrForStamp=timeStrForStamp
        self.separator=separator

    @property
    def __datetime(self):
        return DateTimeParse().parseReturnDateTime(self.timeStrForStamp)

    def customFormat(self,format):
        """
        Return the timestamp in the strftime style defined through 'format'.
        :param format:
        :return:
        """
        return self.__datetime.strftime(format)

    @property
    def tmplMonthAbbrDateHourMin(self):
        """Example: May-04-14-49 for 4th """
        return self.__datetime.strftime(self.separator.join(["%b", "%d", "%H", "%M"]))

    @property
    def tmplHourMin(self):
        """Example: 14-51 for 2:51PM"""
        return self.__datetime.strftime(self.separator.join(["%H", "%M"]))

    @property
    def tmplHourMinSec(self):
        """Example: 14-51 for 2:51PM"""
        return self.__datetime.strftime(self.separator.join(["%H", "%M","%S"]))

    @property
    def tmplYear2dMonthNumDate(self):
        """Example: 21-05-04 for 4th May 2021"""
        return self.__datetime.strftime(self.separator.join(["%y", "%m", "%d"]))

    @property
    def tmplYear4dMonthNumDate(self):
        """Example: 2021-05-05"""
        return self.__datetime.strftime(self.separator.join(["%Y", "%m", "%d"]))

    @property
    def tmplMonthNumDate(self):
        """Example: 05-04 for 4th May"""
        return self.__datetime.strftime(self.separator.join(["%m", "%d"]))


    @property
    def wkDayAbbr(self):
        """
        Weekday as locale’s abbreviated name.

        Sun, Mon, …, Sat (en_US);
        So, Mo, …, Sa (de_DE)
        """
        return self.__datetime.strftime("%a")

    @property
    def wkDayFull(self):
        """Weekday as locale’s full name.

            Sunday, Monday, …, Saturday (en_US);
            Sonntag, Montag, …, Samstag (de_DE)
            """
        return self.__datetime.strftime("%A")

    @property
    def wkDay(self):
        """
        Weekday as a decimal number, where 0 is Sunday and 6 is Saturday.

        0, 1, …, 6"""
        return self.__datetime.strftime("%W")

    @property
    def dayMonth(self):
        """Day of the month as a zero-padded decimal number.


            01, 02, …, 31"""
        return self.__datetime.strftime("%d")

    @property
    def monthAbbr(self):
        """Month as locale’s abbreviated name.

                Jan, Feb, …, Dec (en_US);
                Jan, Feb, …, Dez (de_DE)

        """
        return self.__datetime.strftime("%b")

    @property
    def monthFull(self):
        """Month as locale’s full name.

            January, February, …, December (en_US);
            Januar, Februar, …, Dezember (de_DE)
            """
        return self.__datetime.strftime("%B")

    @property
    def monthNum(self):
        """
            Month as a zero-padded decimal number.
            01, 02, …, 12"""
        return self.__datetime.strftime("%m")

    @property
    def yearWoCent(self):
        """
        Year without century as a zero-padded decimal number.
        00, 01, …, 99"""
        return self.__datetime.strftime("%y")

    @property
    def yearWiCent(self):
        """Year with century as a decimal number.
            0001, 0002, …, 2013, 2014, …, 9998, 9999"""
        return self.__datetime.strftime("%Y")

    @property
    def hour24(self):
        """Hour (24-hour clock) as a zero-padded decimal number.

            00, 01, …, 23"""
        return self.__datetime.strftime("%H")

    @property
    def hour12(self):
        """
            Hour (12-hour clock) as a zero-padded decimal number.
            01, 02, …, 12"""
        return self.__datetime.strftime("%I")

    @property
    def amPm(self):
        """
            Locale’s equivalent of either AM or PM.

            AM, PM (en_US);
            am, pm (de_DE)
        """
        return self.__datetime.strftime("%p")

    @property
    def minuteNum(self):
        """
            Minute as a zero-padded decimal number.

            00, 01, …, 59"""

        return self.__datetime.strftime("%M")

    @property
    def secondNum(self):
        """
            Second as a zero-padded decimal number.

            00, 01, …, 59"""
        return self.__datetime.strftime("%S")

    @property
    def dayOfYearJulian(self):
        """
            Day of the year as a zero-padded decimal number.

            001, 002, …, 366
        """
        return self.__datetime.strftime("%j")

    @property
    def wkNumYearSun(self):
        """
            Week number of the year (Sunday as the first day of the week) as a zero padded decimal number.
            All days in a new year preceding the first Sunday are considered to be in week 0.

            00, 01, …, 53"""
        return self.__datetime.strftime("%U")

    @property
    def wkNumYearMon(self):
        """
            Week number of the year (Monday as the first day of the week) as a decimal number. All days in a new year preceding
            the first Monday are considered to be in week 0.

            00, 01, …, 53"""
        return self.__datetime.strftime("%W")


if __name__ == "__main__":
    pass